#pragma once

namespace TDT4102 {
	struct Color {
        // Colour channel fields
        unsigned char redChannel = 0;
        unsigned char greenChannel = 0;
        unsigned char blueChannel = 0;
        unsigned char alphaChannel = 255; // The alpha channel is used for transparency

        // The easiest way of using this colour class is to use one of the default colours specified above
        // However, you can also specify your own if you so desire. We recommend you use a colour picker for finding the colour you want
        // Many colour picker tools allow you to view values for the red, green, and blue channels, or produce a string of 6 numbers and letters (A to F).
        // Use the first constructor for the former format, where each value must be between 0 and 255
        // You can also specify a transparency value in the same range if you want, where 0 means completely transparent, and 255 means completely opaque
        // If your colour picker produces a 6 character hexadecimal string, use the second constructor instead.

        explicit Color(unsigned char red, unsigned char green, unsigned char blue, unsigned char transparency = 255);
        explicit Color(unsigned int hexadecimalColour);
        explicit Color() = default;

        bool operator!=(Color otherColor);
        bool operator==(Color otherColor);

        // Predefined colour values
        // The first set originates from the Graph_lib library used in the course book
		const static Color transparent;
        const static Color red;
        const static Color blue;
        const static Color green;
        const static Color yellow;
        const static Color white;
        const static Color black;
        const static Color magenta;
        const static Color cyan;
        const static Color dark_red;
        const static Color dark_green;
        const static Color dark_blue;
        const static Color dark_magenta;
        const static Color dark_cyan;
        const static Color mid_gray;
        const static Color dark_gray;
        const static Color light_gray;

        // A second set of colours used from colours used in CSS (intended for web pages)
        // You can find the colour visualised here: https://developer.mozilla.org/en-US/docs/Web/CSS/color_value/color_keywords
        const static Color silver;
        const static Color gray;
        const static Color maroon;
        const static Color purple;
        const static Color fuchsia;
        const static Color lime;
        const static Color olive;
        const static Color navy;
        const static Color teal;
        const static Color aqua;
        const static Color orange;
        const static Color alice_blue;
        const static Color antique_white;
        const static Color aquamarine;
        const static Color azure;
        const static Color beige;
        const static Color bisque;
        const static Color blanched_almond;
        const static Color blue_violet;
        const static Color brown;
        const static Color burly_wood;
        const static Color cadet_blue;
        const static Color chart_reuse;
        const static Color chocolate;
        const static Color coral;
        const static Color cornflower_blue;
        const static Color corn_silk;
        const static Color crimson;
        const static Color dark_goldenrod;
        const static Color dark_grey;
        const static Color dark_khaki;
        const static Color dark_olivegreen;
        const static Color dark_orange;
        const static Color dark_orchid;
        const static Color dark_salmon;
        const static Color dark_seagreen;
        const static Color dark_slateblue;
        const static Color dark_slategray;
        const static Color dark_slategrey;
        const static Color dark_turquoise;
        const static Color dark_violet;
        const static Color deep_pink;
        const static Color deep_skyblue;
        const static Color dim_gray;
        const static Color dim_grey;
        const static Color dodger_blue;
        const static Color firebrick;
        const static Color floral_white;
        const static Color forest_green;
        const static Color gainsboro;
        const static Color ghost_white;
        const static Color gold;
        const static Color goldenrod;
        const static Color green_yellow;
        const static Color grey;
        const static Color honeydew;
        const static Color hot_pink;
        const static Color indian_red;
        const static Color indigo;
        const static Color ivory;
        const static Color khaki;
        const static Color lavender;
        const static Color lavender_blush;
        const static Color lawn_green;
        const static Color lemon_chiffon;
        const static Color light_blue;
        const static Color light_coral;
        const static Color light_cyan;
        const static Color light_goldenrodyellow;
        const static Color light_green;
        const static Color light_grey;
        const static Color light_pink;
        const static Color light_salmon;
        const static Color light_sea_green;
        const static Color light_sky_blue;
        const static Color light_slate_gray;
        const static Color light_slate_grey;
        const static Color light_steel_blue;
        const static Color light_yellow;
        const static Color lime_green;
        const static Color linen;
        const static Color medium_aquamarine;
        const static Color medium_blue;
        const static Color medium_orchid;
        const static Color medium_purple;
        const static Color medium_sea_green;
        const static Color medium_slate_blue;
        const static Color medium_spring_green;
        const static Color medium_turquoise;
        const static Color medium_violet_red;
        const static Color midnight_blue;
        const static Color mint_cream;
        const static Color misty_rose;
        const static Color moccasin;
        const static Color navajo_white;
        const static Color old_lace;
        const static Color olivedrab;
        const static Color orange_red;
        const static Color orchid;
        const static Color pale_goldenrod;
        const static Color pale_green;
        const static Color pale_turquoise;
        const static Color pale_violet_red;
        const static Color papayawhip;
        const static Color peachpuff;
        const static Color peru;
        const static Color pink;
        const static Color plum;
        const static Color powder_blue;
        const static Color rosy_brown;
        const static Color royal_blue;
        const static Color saddle_brown;
        const static Color salmon;
        const static Color sandy_brown;
        const static Color sea_green;
        const static Color sea_shell;
        const static Color sienna;
        const static Color sky_blue;
        const static Color slate_blue;
        const static Color slate_gray;
        const static Color slate_grey;
        const static Color snow;
        const static Color spring_green;
        const static Color steel_blue;
        const static Color tan;
        const static Color thistle;
        const static Color tomato;
        const static Color turquoise;
        const static Color violet;
        const static Color wheat;
        const static Color white_smoke;
        const static Color yellow_green;
        const static Color rebecca_purple;
	};

    inline const TDT4102::Color TDT4102::Color::transparent = [] { return TDT4102::Color{0, 0, 0, 0};}(); 
    inline const TDT4102::Color TDT4102::Color::silver = [] { return TDT4102::Color{0xc0c0c0};}(); 	
    inline const TDT4102::Color TDT4102::Color::black = [] { return TDT4102::Color{0x000000};}(); 	
    inline const TDT4102::Color TDT4102::Color::gray = [] { return TDT4102::Color{0x808080};}(); 	 	
    inline const TDT4102::Color TDT4102::Color::white = [] { return TDT4102::Color{0xffffff};}(); 	
    inline const TDT4102::Color TDT4102::Color::maroon = [] { return TDT4102::Color{0x800000};}(); 	
    inline const TDT4102::Color TDT4102::Color::red = [] { return TDT4102::Color{0xff0000};}(); 	
    inline const TDT4102::Color TDT4102::Color::purple = [] { return TDT4102::Color{0x800080};}(); 	
    inline const TDT4102::Color TDT4102::Color::fuchsia = [] { return TDT4102::Color{0xff00ff};}(); 	
    inline const TDT4102::Color TDT4102::Color::magenta = [] { return TDT4102::Color{0xff00ff};}(); 	
    inline const TDT4102::Color TDT4102::Color::green = [] { return TDT4102::Color{0x008000};}(); 	
    inline const TDT4102::Color TDT4102::Color::lime = [] { return TDT4102::Color{0x00ff00};}(); 	
    inline const TDT4102::Color TDT4102::Color::olive = [] { return TDT4102::Color{0x808000};}(); 	
    inline const TDT4102::Color TDT4102::Color::yellow = [] { return TDT4102::Color{0xffff00};}(); 	
    inline const TDT4102::Color TDT4102::Color::navy = [] { return TDT4102::Color{0x000080};}(); 	
    inline const TDT4102::Color TDT4102::Color::blue = [] { return TDT4102::Color{0x0000ff};}(); 	
    inline const TDT4102::Color TDT4102::Color::teal = [] { return TDT4102::Color{0x008080};}(); 	
    inline const TDT4102::Color TDT4102::Color::aqua = [] { return TDT4102::Color{0x00ffff};}(); 	
    inline const TDT4102::Color TDT4102::Color::orange = [] { return TDT4102::Color{0xffa500};}();
    inline const TDT4102::Color TDT4102::Color::mid_gray = [] { return TDT4102::Color{0x6a6c6e};}(); 	
    inline const TDT4102::Color TDT4102::Color::alice_blue = [] { return TDT4102::Color{0xf0f8ff};}(); 	
    inline const TDT4102::Color TDT4102::Color::antique_white = [] { return TDT4102::Color{0xfaebd7};}(); 	
    inline const TDT4102::Color TDT4102::Color::aquamarine = [] { return TDT4102::Color{0x7fffd4};}(); 	
    inline const TDT4102::Color TDT4102::Color::azure = [] { return TDT4102::Color{0xf0ffff};}(); 	
    inline const TDT4102::Color TDT4102::Color::beige = [] { return TDT4102::Color{0xf5f5dc};}(); 	
    inline const TDT4102::Color TDT4102::Color::bisque = [] { return TDT4102::Color{0xffe4c4};}(); 	
    inline const TDT4102::Color TDT4102::Color::blanched_almond = [] { return TDT4102::Color{0xffebcd};}(); 	
    inline const TDT4102::Color TDT4102::Color::blue_violet = [] { return TDT4102::Color{0x8a2be2};}(); 	
    inline const TDT4102::Color TDT4102::Color::brown = [] { return TDT4102::Color{0xa52a2a};}(); 	
    inline const TDT4102::Color TDT4102::Color::burly_wood = [] { return TDT4102::Color{0xdeb887};}(); 	
    inline const TDT4102::Color TDT4102::Color::cadet_blue = [] { return TDT4102::Color{0x5f9ea0};}(); 	
    inline const TDT4102::Color TDT4102::Color::chart_reuse = [] { return TDT4102::Color{0x7fff00};}(); 	
    inline const TDT4102::Color TDT4102::Color::chocolate = [] { return TDT4102::Color{0xd2691e};}(); 	
    inline const TDT4102::Color TDT4102::Color::coral = [] { return TDT4102::Color{0xff7f50};}(); 	
    inline const TDT4102::Color TDT4102::Color::cornflower_blue = [] { return TDT4102::Color{0x6495ed};}(); 	
    inline const TDT4102::Color TDT4102::Color::corn_silk = [] { return TDT4102::Color{0xfff8dc};}(); 	
    inline const TDT4102::Color TDT4102::Color::crimson = [] { return TDT4102::Color{0xdc143c};}(); 	
    inline const TDT4102::Color TDT4102::Color::cyan = [] { return TDT4102::Color{0x00ffff};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_blue = [] { return TDT4102::Color{0x00008b};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_cyan = [] { return TDT4102::Color{0x008b8b};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_goldenrod = [] { return TDT4102::Color{0xb8860b};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_gray = [] { return TDT4102::Color{0xa9a9a9};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_green = [] { return TDT4102::Color{0x006400};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_grey = [] { return TDT4102::Color{0xa9a9a9};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_khaki = [] { return TDT4102::Color{0xbdb76b};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_magenta = [] { return TDT4102::Color{0x8b008b};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_olivegreen = [] { return TDT4102::Color{0x556b2f};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_orange = [] { return TDT4102::Color{0xff8c00};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_orchid = [] { return TDT4102::Color{0x9932cc};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_red = [] { return TDT4102::Color{0x8b0000};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_salmon = [] { return TDT4102::Color{0xe9967a};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_seagreen = [] { return TDT4102::Color{0x8fbc8f};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_slateblue = [] { return TDT4102::Color{0x483d8b};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_slategray = [] { return TDT4102::Color{0x2f4f4f};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_slategrey = [] { return TDT4102::Color{0x2f4f4f};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_turquoise = [] { return TDT4102::Color{0x00ced1};}(); 	
    inline const TDT4102::Color TDT4102::Color::dark_violet = [] { return TDT4102::Color{0x9400d3};}(); 	
    inline const TDT4102::Color TDT4102::Color::deep_pink = [] { return TDT4102::Color{0xff1493};}(); 	
    inline const TDT4102::Color TDT4102::Color::deep_skyblue = [] { return TDT4102::Color{0x00bfff};}(); 	
    inline const TDT4102::Color TDT4102::Color::dim_gray = [] { return TDT4102::Color{0x696969};}(); 	
    inline const TDT4102::Color TDT4102::Color::dim_grey = [] { return TDT4102::Color{0x696969};}(); 	
    inline const TDT4102::Color TDT4102::Color::dodger_blue = [] { return TDT4102::Color{0x1e90ff};}(); 	
    inline const TDT4102::Color TDT4102::Color::firebrick = [] { return TDT4102::Color{0xb22222};}(); 	
    inline const TDT4102::Color TDT4102::Color::floral_white = [] { return TDT4102::Color{0xfffaf0};}(); 	
    inline const TDT4102::Color TDT4102::Color::forest_green = [] { return TDT4102::Color{0x228b22};}(); 	
    inline const TDT4102::Color TDT4102::Color::gainsboro = [] { return TDT4102::Color{0xdcdcdc};}(); 	
    inline const TDT4102::Color TDT4102::Color::ghost_white = [] { return TDT4102::Color{0xf8f8ff};}(); 	
    inline const TDT4102::Color TDT4102::Color::gold = [] { return TDT4102::Color{0xffd700};}(); 	
    inline const TDT4102::Color TDT4102::Color::goldenrod = [] { return TDT4102::Color{0xdaa520};}(); 	
    inline const TDT4102::Color TDT4102::Color::green_yellow = [] { return TDT4102::Color{0xadff2f};}(); 	
    inline const TDT4102::Color TDT4102::Color::grey = [] { return TDT4102::Color{0x808080};}(); 	
    inline const TDT4102::Color TDT4102::Color::honeydew = [] { return TDT4102::Color{0xf0fff0};}(); 	
    inline const TDT4102::Color TDT4102::Color::hot_pink = [] { return TDT4102::Color{0xff69b4};}(); 	
    inline const TDT4102::Color TDT4102::Color::indian_red = [] { return TDT4102::Color{0xcd5c5c};}(); 	
    inline const TDT4102::Color TDT4102::Color::indigo = [] { return TDT4102::Color{0x4b0082};}(); 	
    inline const TDT4102::Color TDT4102::Color::ivory = [] { return TDT4102::Color{0xfffff0};}(); 	
    inline const TDT4102::Color TDT4102::Color::khaki = [] { return TDT4102::Color{0xf0e68c};}(); 	
    inline const TDT4102::Color TDT4102::Color::lavender = [] { return TDT4102::Color{0xe6e6fa};}(); 	
    inline const TDT4102::Color TDT4102::Color::lavender_blush = [] { return TDT4102::Color{0xfff0f5};}(); 	
    inline const TDT4102::Color TDT4102::Color::lawn_green = [] { return TDT4102::Color{0x7cfc00};}(); 	
    inline const TDT4102::Color TDT4102::Color::lemon_chiffon = [] { return TDT4102::Color{0xfffacd};}(); 	
    inline const TDT4102::Color TDT4102::Color::light_blue = [] { return TDT4102::Color{0xadd8e6};}(); 	
    inline const TDT4102::Color TDT4102::Color::light_coral = [] { return TDT4102::Color{0xf08080};}(); 	
    inline const TDT4102::Color TDT4102::Color::light_cyan = [] { return TDT4102::Color{0xe0ffff};}(); 	
    inline const TDT4102::Color TDT4102::Color::light_goldenrodyellow = [] { return TDT4102::Color{0xfafad2};}(); 	
    inline const TDT4102::Color TDT4102::Color::light_gray = [] { return TDT4102::Color{0xd3d3d3};}(); 	
    inline const TDT4102::Color TDT4102::Color::light_green = [] { return TDT4102::Color{0x90ee90};}(); 	
    inline const TDT4102::Color TDT4102::Color::light_grey = [] { return TDT4102::Color{0xd3d3d3};}(); 	
    inline const TDT4102::Color TDT4102::Color::light_pink = [] { return TDT4102::Color{0xffb6c1};}(); 	
    inline const TDT4102::Color TDT4102::Color::light_salmon = [] { return TDT4102::Color{0xffa07a};}(); 	
    inline const TDT4102::Color TDT4102::Color::light_sea_green = [] { return TDT4102::Color{0x20b2aa};}();
    inline const TDT4102::Color TDT4102::Color::light_sky_blue = [] { return TDT4102::Color{0x87cefa};}();
    inline const TDT4102::Color TDT4102::Color::light_slate_gray = [] { return TDT4102::Color{0x778899};}();
    inline const TDT4102::Color TDT4102::Color::light_slate_grey = [] { return TDT4102::Color{0x778899};}();
    inline const TDT4102::Color TDT4102::Color::light_steel_blue = [] { return TDT4102::Color{0xb0c4de};}();
    inline const TDT4102::Color TDT4102::Color::light_yellow = [] { return TDT4102::Color{0xffffe0};}(); 	
    inline const TDT4102::Color TDT4102::Color::lime_green = [] { return TDT4102::Color{0x32cd32};}(); 	
    inline const TDT4102::Color TDT4102::Color::linen = [] { return TDT4102::Color{0xfaf0e6};}();
    inline const TDT4102::Color TDT4102::Color::medium_aquamarine = [] { return TDT4102::Color{0x66cdaa};}(); 	
    inline const TDT4102::Color TDT4102::Color::medium_blue = [] { return TDT4102::Color{0x0000cd};}(); 	
    inline const TDT4102::Color TDT4102::Color::medium_orchid = [] { return TDT4102::Color{0xba55d3};}(); 	
    inline const TDT4102::Color TDT4102::Color::medium_purple = [] { return TDT4102::Color{0x9370db};}(); 	
    inline const TDT4102::Color TDT4102::Color::medium_sea_green = [] { return TDT4102::Color{0x3cb371};}();
    inline const TDT4102::Color TDT4102::Color::medium_slate_blue = [] { return TDT4102::Color{0x7b68ee};}();
    inline const TDT4102::Color TDT4102::Color::medium_spring_green = [] { return TDT4102::Color{0x00fa9a};}();
    inline const TDT4102::Color TDT4102::Color::medium_turquoise = [] { return TDT4102::Color{0x48d1cc};}(); 	
    inline const TDT4102::Color TDT4102::Color::medium_violet_red = [] { return TDT4102::Color{0xc71585};}();
    inline const TDT4102::Color TDT4102::Color::midnight_blue = [] { return TDT4102::Color{0x191970};}(); 	
    inline const TDT4102::Color TDT4102::Color::mint_cream = [] { return TDT4102::Color{0xf5fffa};}(); 	
    inline const TDT4102::Color TDT4102::Color::misty_rose = [] { return TDT4102::Color{0xffe4e1};}(); 	
    inline const TDT4102::Color TDT4102::Color::moccasin = [] { return TDT4102::Color{0xffe4b5};}(); 	
    inline const TDT4102::Color TDT4102::Color::navajo_white = [] { return TDT4102::Color{0xffdead};}(); 	
    inline const TDT4102::Color TDT4102::Color::old_lace = [] { return TDT4102::Color{0xfdf5e6};}(); 	
    inline const TDT4102::Color TDT4102::Color::olivedrab = [] { return TDT4102::Color{0x6b8e23};}(); 	
    inline const TDT4102::Color TDT4102::Color::orange_red = [] { return TDT4102::Color{0xff4500};}(); 	
    inline const TDT4102::Color TDT4102::Color::orchid = [] { return TDT4102::Color{0xda70d6};}(); 	
    inline const TDT4102::Color TDT4102::Color::pale_goldenrod = [] { return TDT4102::Color{0xeee8aa};}(); 	
    inline const TDT4102::Color TDT4102::Color::pale_green = [] { return TDT4102::Color{0x98fb98};}(); 	
    inline const TDT4102::Color TDT4102::Color::pale_turquoise = [] { return TDT4102::Color{0xafeeee};}(); 	
    inline const TDT4102::Color TDT4102::Color::pale_violet_red = [] { return TDT4102::Color{0xdb7093};}();
    inline const TDT4102::Color TDT4102::Color::papayawhip = [] { return TDT4102::Color{0xffefd5};}(); 	
    inline const TDT4102::Color TDT4102::Color::peachpuff = [] { return TDT4102::Color{0xffdab9};}(); 	
    inline const TDT4102::Color TDT4102::Color::peru = [] { return TDT4102::Color{0xcd853f};}(); 	
    inline const TDT4102::Color TDT4102::Color::pink = [] { return TDT4102::Color{0xffc0cb};}(); 	
    inline const TDT4102::Color TDT4102::Color::plum = [] { return TDT4102::Color{0xdda0dd};}(); 	
    inline const TDT4102::Color TDT4102::Color::powder_blue = [] { return TDT4102::Color{0xb0e0e6};}(); 	
    inline const TDT4102::Color TDT4102::Color::rosy_brown = [] { return TDT4102::Color{0xbc8f8f};}(); 	
    inline const TDT4102::Color TDT4102::Color::royal_blue = [] { return TDT4102::Color{0x4169e1};}(); 	
    inline const TDT4102::Color TDT4102::Color::saddle_brown = [] { return TDT4102::Color{0x8b4513};}(); 	
    inline const TDT4102::Color TDT4102::Color::salmon = [] { return TDT4102::Color{0xfa8072};}(); 	
    inline const TDT4102::Color TDT4102::Color::sandy_brown = [] { return TDT4102::Color{0xf4a460};}(); 	
    inline const TDT4102::Color TDT4102::Color::sea_green = [] { return TDT4102::Color{0x2e8b57};}(); 	
    inline const TDT4102::Color TDT4102::Color::sea_shell = [] { return TDT4102::Color{0xfff5ee};}(); 	
    inline const TDT4102::Color TDT4102::Color::sienna = [] { return TDT4102::Color{0xa0522d};}(); 	
    inline const TDT4102::Color TDT4102::Color::sky_blue = [] { return TDT4102::Color{0x87ceeb};}(); 	
    inline const TDT4102::Color TDT4102::Color::slate_blue = [] { return TDT4102::Color{0x6a5acd};}(); 	
    inline const TDT4102::Color TDT4102::Color::slate_gray = [] { return TDT4102::Color{0x708090};}(); 	
    inline const TDT4102::Color TDT4102::Color::slate_grey = [] { return TDT4102::Color{0x708090};}(); 	
    inline const TDT4102::Color TDT4102::Color::snow = [] { return TDT4102::Color{0xfffafa};}(); 	
    inline const TDT4102::Color TDT4102::Color::spring_green = [] { return TDT4102::Color{0x00ff7f};}(); 	
    inline const TDT4102::Color TDT4102::Color::steel_blue = [] { return TDT4102::Color{0x4682b4};}(); 	
    inline const TDT4102::Color TDT4102::Color::tan = [] { return TDT4102::Color{0xd2b48c};}(); 	
    inline const TDT4102::Color TDT4102::Color::thistle = [] { return TDT4102::Color{0xd8bfd8};}(); 	
    inline const TDT4102::Color TDT4102::Color::tomato = [] { return TDT4102::Color{0xff6347};}(); 	
    inline const TDT4102::Color TDT4102::Color::turquoise = [] { return TDT4102::Color{0x40e0d0};}(); 	
    inline const TDT4102::Color TDT4102::Color::violet = [] { return TDT4102::Color{0xee82ee};}(); 	
    inline const TDT4102::Color TDT4102::Color::wheat = [] { return TDT4102::Color{0xf5deb3};}(); 	
    inline const TDT4102::Color TDT4102::Color::white_smoke = [] { return TDT4102::Color{0xf5f5f5};}(); 	
    inline const TDT4102::Color TDT4102::Color::yellow_green = [] { return TDT4102::Color{0x9acd32};}(); 	
    inline const TDT4102::Color TDT4102::Color::rebecca_purple = [] { return TDT4102::Color{0x663399};}();
}

