// Lecture 4, Example 2 - Dragana Laketic

#include "std_lib_facilities.h"
#include "AnimationWindow.h"

// This file provides examples of how to use vector and array in C++

// Repetition:
// We can call functions in main() only if they are declared before the first call.
// C++ compilers work from top towards the bottom of the file which means that declarations
// of the functions which will be used in main() must be provided at this place.
// Every definition is declaration (not the other way round - check the pensum book!) so
// either declaration or definition must be provided before the first function invocation in main().
//
// Below is the definition of the function which prints the vector of integers.
// 
void printVector(vector<int> v) {
    for (int i = 0; i < v.size(); i++) {
        cout << v.at(i) << " ";
    }
    cout << endl;
}
// ... and here is the declaration of the function which prints an array of 5 integers.
// Mark that the definition of this function is provided after the main()!
void printArray(array<int, 5> a);

int main() {
    // VECTOR
    vector<int> vec{1, 2, 3}; // definition of a variable of type vector<int>
			      // We use the keyword vector followed by the data type
			      // of the elements of the vector within <>.
			      // Mark that all the elements of one vector are of the same data type.
			      // Variable vec is initialised so as to contain three integer values: 1, 2, 3

    cout << "Original vector: ";
    printVector(vec);
    // The size of the vector is the number of the elements it contains.
    // We can get it by calling the function size() on the vector variable:
    cout << "The size of the original vector: " << vec.size() << endl;
    // Elements of the vector are indexed from 0. That means that the first element
    // has index 0 and the last element has index size()-1.
    cout << "The second element is: " << vec.at(1) << endl;
    // To access the i-th element of the vector we use the function at() on the vector variable.
    // Function at() also checks whether the index passed to it is within the limits of the vector
    // variable: If we try to access element outside of the vector limits, compiler will prevent
    // us from doing so by issuing an error. 
    vec.at(2) += vec.at(0) + vec.at(1);
    cout << "The sum of the elements stored in the last element: " << vec.at(2) << endl;
    // Vectors can change the size. In the line below the size of the vector is changed by
    // adding the integer element of value 5 to the end of the vector variable. This is
    // achieved by calling the function push_back() on the vector variable.
    vec.push_back(5);
    cout << "expanded vector: ";
    printVector(vec);
    cout << "The size of the expanded vector: " << vec.size() << endl;
    // Another way to change the size of the vector is to call the function resize()
    // on the vector variable. We need to pass the new size as the parameter to function resize().
    // In the following example the new size of the vector variable will be 8.
    // Mark that the newly added elements are initialised to value 0 which is the default
    // initial value for integers.
    vec.resize(8);
    cout << "Vector after resizing to 8 elements: ";
    printVector(vec);
    // If we resize the vector variable to a size smaller than its current size, elements
    // beyond that size will simply be cut off!
    vec.resize(2);
    cout << "Vector after resizing to 2 elements: ";
    printVector(vec);

    // ARRAY
    array<int, 5> arr{1, 2, 3}; // definition of a variable of type array<int, 5>.
				// It is similar to the definition of a vector with
				// the difference that we need to provide the size of
				// the array - which is 5 in this example. And this is
				// the difference between vector and array variables:
				// array variable cannot change its size like vectors.
				// Variable arr will always have 5 elements. The first three
				// are initialised to 1, 2 and 3 and the rest is 0.
    cout << "Original array: ";
    printArray(arr);
    // Functions size() and at() work on array variable in the same way as on vector variable:
    cout << "The first element is: " << arr.at(0) << " and the last: " << arr.at(arr.size()-1) << endl;
    // Uncomment the code line below - at() does the check of the array limits!
    // cout << arr.at(7); // error
    return 0;
}

// The definition of the function to print an array variable of 5 elements.
// We can use this function in the main() above because it is declared at
// the top of the file. 
void printArray(array<int, 5> a) {
    for (int i = 0; i < 5; i++) {
        cout << a.at(i) << " ";
    }
    cout << endl;
}

// Another way of addressing elements of vector and array variables is with index-operator [].
// This way of addressing the elements does not perform check for the limits [0, size()) which
// means that we can get some unexpected results. Run the examples from slides 19 and 20 from
// the lecture. What is the explanation for such behaviour?
