// Lecture 4, Example 3 - Dragana Laketic

#include "std_lib_facilities.h"

// PSEUDORANDOM GENERATOR
// Classical computers we use today are deterministic machines so
// they cannot produce truly random numbers like, for example, nature can.
// But often there is a need for random numbers when we write programs -
// think of programming games, simulations of scientific experiments or many other
// examples. We can only make computers generate sequences of numbers which appear
// to be random, i.e., hard to predict what the next generated value will be if
// we know the values previously generated. We call such sequences pseudonumbers.
// Usually there is a simple algorithm to generate these sequences (look at the example
// from the slide 30 from the lecture!) which are started by some initial value called
// "seed" and then used to generate each new value by feeding the previous output
// to its input where the seed was used at the first step. In that fashion, for the same
// seed the pseudogenerator can always generate the same pseudorandom sequence. Think how
// useful it can be for reproducing hte results of some experiment!
 
int main() {
    // Here is the example of a pseudonumber generator from the C++ standard library
    random_device dev; // (pseudo)random number ...
    unsigned int rndSeed = dev(); // ... here generated and assigned to a variable which
				  // will be used as a seed to a (pseudo)random number generator
				  // as in the example commented in the line below
    //default_random_engine engine(rndSeed);
    default_random_engine engine(7); // and here is the fixed value of the seed for which
				     // the generated (pseudo)random values will always be the samesequence
    uniform_int_distribution dstr(0, 100); // dstr variable is generated so as to satisfy uniform distribution
					   // of integer values in the interval [0, 100)
    
    // Run the following code several times with engine variable being set to random seed and to a fixed value.
    // Describe the sequences you get in both cases.
    cout << "The first five pseudorandom integers: ";
    for (int i = 0; i < 5; i++) {
        cout << dstr(engine) << " ";
    }
    cout << endl;

    return 0;
}
