// Lecture 6, Example, repetition - Dragana Laketic

#include "std_lib_facilities.h"

// This example is a bit more elaborate answer to a question which came up
// during the last lecture: what is the difference between i++ and ++i?
// It is also used to refresh on the new knowledge we gained last week
// on important container from the C++ standard library: vector.
//
// Both i++ and ++i increment the value of variable i by 1.
// If, for example, variable i contained value 7, after any of the two statements
// i++; // postfix notation
// ++i; // prefix notation
// are executed the value of the variable i will be 8.
// The different results may come out when we use these expressions within
// more complex statements. Here we use a simple example where a variable i
// which is used as a control variable within a for-loop is incremented within
// the expression which calls the function at() on the vector to access its
// i-th element.
//
// Remember:
// 1) Be aware of how you change the loop control variable! It may have side effects
// which can cause all sorts of errors.
// 2) Use at() to access elements of vector and array variables!
// 3) Avoid using [] for that purpose although you may be used to it if you work(ed) with Python
// 4) Compilers may vary at how they implement coplex statements, like vec.at(++i).
// Avoid taking chances unless you know your compiler very very well ... and even then!
//
// Run each version of the functions printVectorH1() and printVectorH2() and
// explain the result of the program. Functions for printing vectors are similar
// to those we used in the previous lecture. 

// Version 1
// In this case we use the most common version of for-loop where the value of the
// control variable (i) is changed in the third statement of the for-loop. Remember that
// this statement is executed at the end of the for-loop block, just before the condition (the second
// statement of the for-loop) is checked in order to determine whether the next iteration will
// be performed. The same control variable for the for-loop is used as an index of the elements
// of the vector. 
// Remember: elements of the vector are index-ed from 0 so that the last element
// has index vec_variable.size()-1. Function .size() returns the number of elements
// in the vector and function .at() is used to access its elements. This function
// also checks the validity of the index it gets passed as a parameter whether it points
// within the limits of the memory assigned to the vector.

/* Version 1 - BEGIN  
void printVectorH1(vector<char> vv) {
    for (int i = 0; i < vv.size(); i++) {
        cout << vv.at(i) << "  ";
    }
    cout << endl;
}

void printVectorH2(vector<char> vv) {
    for (int i = 0; i < 7; ++i) {
        cout << vv.at(i) << "  ";
    }
    cout << endl;
}
Version 1 - END */

// Version 2
// In this version we change the value of the control variable i, which we also
// use as an index to vector elements, within the call to function at() on the vector.
// Postfix variant in printVectorH1() should first call function at.() with
// the value of variable i (0 in the first iteration) and then execute i++ which
// increments the value of i by 1 (1 after this line of code: cout << vv.at(i++) << "  ";)
// Prefix variant in printVectorH2() should first increment the value of i (in the first
// iteration it will be the change from 0 to 1) and then call the function at(). This means
// that in the first iteration we shall access the second element of the vector. And so forth
// until we come to the last iteration: the value of i will be less than vector's size but
// when increased by 1 before calling at() on the vector it will cause an error of
// attempting an illegal access. Remember: at() checks the allowed range of indices!
// Also, be aware that not all compilers interpret the more complex expressions in the same way.
// If you can, try running the same code with different compilers and/or on different platforms.
// The results and the errors raised may differ quite a lot. 

/* Version 2 - BEGIN
void printVectorH1(vector<char> vv) {
    for (int i = 0; i < vv.size();) {
        cout << vv.at(i++) << "  ";
    }
    cout << endl;
}

void printVectorH2(vector<char> vv) {
    for (int i = 0; i < 7;) {
        cout << vv.at(++i) << "  ";
    }
    cout << endl;
}
Version 2 - END */

// Version 3
// This is example to remind you of the pitfalls of using [] to address
// the elements of the vector (or the array).

/* Version 3 - BEGIN
void printVectorH1(vector<char> vv) {
    for (int i = 0; i < vv.size();) {
        cout << vv[i++] << "  ";
    }
    cout << endl;
}

void printVectorH2(vector<char> vv) {
    for (int i = 0; i < 7;) {
        cout << vv[++i] << "  ";
    }
    cout << endl;
}
Version 3 - END */

int main() {
    vector<char> v{'a', 'b', 'c'};

    printVectorH1(v);
    printVectorH2(v);

    return 0;
}
