#include "std_lib_facilities.h"

// Repetition: struct is a type of data structure which contains
// several variables. Variables can be of different types. In the example
// below we declare a struct named 'Lake' which contains three variables:
// a string name, a string moon and a real number length.
// As a rule, we kepp such declarations in a separate header file,
// for example lake.h, and then include that file in all files which
// need to use the struct:
// #include "Lake.h"
// In this case we shall keep it in the same file where our main function is.
struct Lake {
    string name;
    string moon;
    double length;
};

// By default, the members of a struct have visibility 'public'.
// Therefore, the following declaration is equivalent to the
// declaration above:
/* Example of use of keyword 'public'
struct Lake {
public:
    string name;
    string moon;
    double length;
};
*/

// If a member of a struct is declared 'private', it cannot be
// accessed from a function which is not a public method
// (a public member function) of the struct.
// Declaration below is a modified version of 'struct Lake'
// which also contains a private member: string planet.
// We cannot reach this member from any function which is not
// declared as a member function of 'struct Lake' (or a friend
// function but that will be talked about in few lectures ahead!)
/*
struct Lake {
public:
    string name;
    string moon;
    double length;
private:
    string planet;
};
*/

// Objects of type 'struct' cannot be printed to the terminal as
// a whole since operator '<<' does not know how to print members of
// the 'struct'. But, we can define a function which will print an object
// of type struct Lake in a way we want. Note that we access the data
// members of the variable of type Lake by dot notation:
// <name_of_struct_variable>.<name_of_member> and that these members
// have public visibility.
void printLake(Lake ll) {
    cout << "Lake " << ll.name << "  on " << ll.moon << endl;
    // Uncomment the line below and the third declaration of 'struct Lake',
    // the one which contains variable 'string planet'. This variable
    // has visibility 'private' and will not be possible to be accessed
    // from this function!
    // cout << "Orbiting planet " << ll.planet << endl;
}

int main() {
    Lake oh; // Declaration of variable oh which is of the type Lake (struct).

    oh.name = "Ohrid"; // Assignment of value to the data member 'name' of 'oh' variable.
    oh.moon = "Titan"; // Assignment of value to the data member 'moon' of 'oh' variable.
    oh.length = 17.3; // Assignment of value to the data member 'length' of 'oh' variable.

    // We can write values of public data members of a variable of type struct
    cout << "Hello from lake " << oh.name << " which is on " << oh.moon << "  " <<
    oh.length << " long" << endl;
    // ... but not the private data members!
    // Uncomment the line below together with the declaration of struct Lake which
    // contains private element 'string planet'. It cannot be accessed from this function!
    //cout << "Orbiting " << oh.planet << endl;

    printLake(oh); // Example of a function call which takes variable of type 'struct'

    return 0;
}
