#pragma once
#include "std_lib_facilities.h"

// We keep class declarations in a header file like this.
// This header file contains only one class declaration and is
// conveniently named by the same name as the class. If there are
// more classes which logically belong together, i.e., which are
// usually used together, we can group them in the same header file.
// In this way we keep the code tidy, maintain readability and also
// make it easy to use it where needed by simply including the needed
// header.

// Example of class declaration:
// keyword 'class' followed by the name of the class, 'Mission',
// and class members between the braces. Class members can be
// data members (data type followed by a variable name) and member functions,
// also called methods, (declaration of a function).
class Mission {
private:
    // The following class members have 'private' visibility.
    // They cannot be accesses outside from the class - by which
    // we mean by any of the function which is not a member function
    // of this class. (Well, in a few lecures ahead you will see
    // that it is possible for special functions we call friends but
    // don't think of it for the time being!).
    string name; // Example of data member named 'name' of type 'string'.
    int year; // Example of data memebr named 'year' of type 'int'.
    int yearssincelaunch; // Yet another data member of type 'int', it's name 'yearssincelaunch'.
public:
    // The following class members are 'public' which means that
    // they can be accessed from any place in the program where we need
    // to perform certain operation on the data members of the object
    // which is an instance of this class.

    // The following two declarations are declarations of class constructors.
    // When an object of the class type is created somewhere in the computer
    // memory at the time it is instantiated somewhere in the program, a special
    // function called 'constructor' is called. The name speaks for itself - this
    // function constructs an object in the memory. It sets the space for data members,
    // initialises data members as specified by the constructor-function called, and
    // ensures that the member fnctions will be possible to be called for the object
    // with the corresponding access to its data.
    // Constructors are special type of functions: they have no return data type and
    // their name is the same as the name of the class.
    // As we know, it is possible to overload functions and so it holds also for
    // constructors - the only difference is in the list of parameters passed to it.
    Mission(); // This constructor is called in case no data is provided
	       // when we declare a variable of type 'Mission'. See the example
	       // of variable 'enannen' in the main-function.
    Mission(string nn, int yr); // This constructor is called when we provide two values:
				// one string and one int, to initialise variable of type
				// 'Mission'. See the example of variable 'cas' in the main-function.
    void printMission(); // This is example of member function or a method of class Mission.
			 // This function has access to private members of the object of type Mission.
			 // See the definition of this function in Mission.cpp file.
};

// Visibility of class members is by default private so the declaration above
// is equivalent to the declaration below:
/*
class Mission {
    string name;
    int year;
    int yearssincelaunch;
public:
    Mission();
    Mission(string nn, int yr);
    void printMission();
};
*/
