// Lecture 8, Example 3 - Dragana Laketic

//#include "std_lib_facilities.h"
#include <unordered_map>
#include <map>
#include <string>
#include <iostream>

// Maps in C++ are data structures which keep data elements in pairs
// {key, value}, like dictionaries in Python. There are two types of maps
// in C++: map and unordered_map. Use of map assumes that the elements
// kept in it are sorted by the key value, while unordered_map does not
// take care of keeping its elements sorted.
// In order to use this data structure, we have to include corresponding
// header - look at the first two include statements in this file!

    // Below is the definition of a helper function to print the
    // elements of a map. (We should keep its definition in a separate
    // .cpp file and the declaration in the corresponding .h file then include
    // that header in this file in order to use the function here but we'll
    // keep the definition here for a better overview).
    // Mark how we pass by reference the parameter to the function and
    // in that way avoid copying of the whole map we want to print
    // (these can often be quite big).
void printTheatres(std::map<std::string, unsigned int>& refMap) {
        // Example of a range-based loop: it is convenient for
        // traversing the whole map.
    for(const std::pair<std::string, unsigned int> p : refMap) {
            // Map consists of pairs {key, value} of a correspondig type pair.
            // Variable p, visible in the body of this for-loop represents
            // a {key, value} element of the map refMap. Mark how p is declared
            // with the same type specifiers within <,> just like the map
            // it is used to traverse.
        std::cout << p.first << " har " << p.second << " stk teater" << std::endl;
            // The line above shows that the first lement of the pair, which
            // corresponds to the key of the map's pair, is obtained by p.first
            // and the second element which corresponds to the value of that key
            // by p.second.
    }
}

int main() {
        // Compare map (ordered) and unordered_map
    //std::unordered_map<std::string, unsigned int> stat;
    std::map<std::string, unsigned int> stat; // Declaration of a variable of type map.
                                            // After keyword map we need to specify
                                            // the type of the key and of the value
                                            // between "<" and ">" (in a few weeks
                                            // you will understand why we use this notation).
    stat.insert({"Trondheim", 3}); // New element / pair of {key, value} is inserted into
                                // a map by calling a function insert().
    stat.insert({"Aalesund", 1});
    stat.insert({"Oslo", 20});
    stat.insert({"Bergen", 3});
    printTheatres(stat);
    
        // Similar to members of vectors and arrays we can address
        // the members of a map by the key by using either function at()
        // or operator []. But be cautious with using []!!!
    if (stat.at("Trondheim") > 1) {
        std::cout << "Vi er teaterbyen!" << std::endl;
    }
    printTheatres(stat);
    
        // If we use operator [] to address the element of map by the key
        // and if the element with such key does not exist, it will be inserted
        // in the map!!!
    if (stat["Larvik"] > 1) {
        std::cout << "Larvik er teaterbyen!" << std::endl;
    }
    printTheatres(stat);

    return 0;
}
