// Lecture 10, Examples 1-6, Dragana Laketic

#include "std_lib_facilities.h"

// Memory
// Pointers
// - operators '&' and '*'
// Scope
// Dynamic memory allocation
// - operators 'new' and 'delete', 'new[]' and 'delete[]'
// Destructors
// Copy constructors and copy assignments

	// Definition of a simple class which is used
	// in the examples to show how pointers to
	// user-defined types are used.
	// The class contains one provate member of type int,
	// one default constructor, one constructor which takes
	// one int parameter and initialises its member stat to
	// the value of this parameter, and two functions which
	// allow user of the class to set the stat value (setStat())
	// and to read it (getStat).
class Qbit {
    int stat;
public:
    Qbit() : stat{0} {}
    Qbit(int v) : stat{v} {}
    void setStat(int v) {stat = v;}
    int getStat(void) const {return stat;}
};

    // Definition of a class which is used in the examples to
    // demonstrate why we need destructors (5)), copy constructors (6))
    // and to overload the assignment operator (6a)).
class Cnot {
    // NOTE! this member of the class is a pointer. Most likely
    // use of pointers is for dynamical allocations of memory.
    // In the example of this class, it is a pointer to an object
    // of type class Qbit which is dynamically allocated when the
    // object is initialised in the memory - see how the constructors
    // are defined!
    Qbit* qb;
public:
    Cnot()  {
        // Default constructor:
        // Dynamically allocates an object of type Qbit and
        // assigns its address to its data member 'qb' which is
        // of type 'pointer to object of type Qbit'
        qb = new Qbit;
    }
    Cnot(const Cnot& cv) {
        // Copy constructor:
        // This constructor is used when we instantiate a new object
        // of type class Cnot and initialise it with another object of
        // the same type. Note how we pass the object with which we want
        // the new object initialised - or constructed with - by reference
        // so we avoid copying it and how we use const qualifier to secure
        // that it will not be changed within the constructor,

            // Dynamical allocation of memory which stores the object
            // of type Qbit. This memory will be held py pointer qb as
            // long as an object of type class Cnot is valid (in scope
            // of the program where it is used).
        qb = new Qbit;
            // Since we initialise new object with 'cv' object, we want
            // to know what value is stored in its dynamically allocated
            // memory address of which is kept in its data member 'cv.qb'.
            // This kind of copying is called 'deep copy' since we copy
            // the values the pointers point to. Contrary to it, just
            // copying pointers (qb = cv.qb;) would copy the address stored
            // in cv.qb and would lead to the case where newly created
            // object of type Cnot and object cv with which we initialise it
            // both point to the same memory location and this is not what
            // we want! This kind of copying is calles 'shallow copy'.
        int les = cv.qb->getStat();
        cout << "Innen kopikonstruktoer!" << endl;
            // Here we assign the value stored in memory location pointed
            // to by cv.qb to the memory newly allocated in this copy
            // constructor of which the address is kept in data member qb.
        qb->setStat(les);
    }
    // Destructors are special functions which are called when an object
    // of a user defined type goes out of scope and is destroyed in
    // the memory. It is the place where the memory is 'tidied' after
    // being used by the object members.
    // Destructors have a special syntax: they have no return value and
    // they take no parameters, their name begins with '~'followed immediately
    // by the name of the class.
    // Contrary to constructors for which we can have several overloaded
    // versions, there can be only one destructor per class!
    // Destructors exist for all objects even when not explicitly defined -
    // the cases when they are generated by compiler and when they
    // 'clean' some members of built-in types. However, when we have a class
    // with dynamically allocated members, we need to make it explicit
    // how to clean the members of the objects of such class types - as
    // in the example below:
    ~Cnot() {
        cout << "Innen destruktoer!" << endl;
            // Let the OS know that it can release the memory
            // it allocated for the object in the constructor.
            // If the destructor is not defined so as to ask
            // for deallocation of the memory resource, the OS
            // would keep that memory unavailable for other purposes
            // and we would cause memory leakage every time
            // an object of type Cnot goes out of scope.
            // Lesson to take:
            // Whenever you have members of the class dynamically allocated,
            // define a destructor which releases that memory. Or, follow
            // the rule: foe every new / new[] in the constructor,
            // provide a matching delete / delete[] in the destructor.
        delete qb;
    }
        // Member functions which provide the user with the possibility
        // to read and write the value stored in the object pointed to
        // by its member qb.
    int readQb() {return qb->getStat();}
    void writeQb(int v) {qb->setStat(v);}

    // When a class has dynamically allocated resources (objects placed
    // on heap memory and obtained from OS system via operator new / new[]),
    // it is a good practive to define overloaded operator= which will
    // ensure that copy assignment is performed correctly, i.e.,
    // if we assign one object of the class type to another,
    // the dynamically allocated resources will be deep-copied. In other
    // words, we do not copy the values of the pointers, rather the values
    // to which they point to.
    // Below is an example of an overloaded opertor= for the class Cnot.
    // You may notice that it is quite similar to the copy constructor.
    // Usually, it is the case for these two special functions. The difference
    // is mainly in the fact that when we assign an object to another
    // object, they are already created - instantiated in the memory
    // and it may happen that the pointer points to some valid
    // address in the memory. Therefore, we first test if that is the case
    // and ask OS for memory allocation only if it is not.
    Cnot& operator=(const Cnot& cnot) {
        cout << "Innen overlastet operator=!" << endl;
        int les = cnot.qb->getStat();
            // Check if the member pointer points to a valid address:
        if(qb == nullptr) {
                // and if not, as OS to allocate object
                // of type Qbit so that its address is
                // stored in pointer qb.
            qb = new Qbit;
        }
            // Assign the value which was read from the object
            // cnot (the value stored in the object Qbit to which
            // the pointer member qb of object cnot points to)
            // to this object:
        qb->setStat(les);

            // 'this' is a pointer to the instantiated object
            // and is present for every instantiated object of
            // any class. (Similar to 'self' in Python). Note
            // how we use dereferenced pointer this to return
            // exactly the object for which we call this overloaded
            // operator so that it can be used in the way commonly
            // accepted for the assignment operator:
            // <object_1> = <object_2>
        return *this;
    }
};

    // Definition of function which is used as an example 3)
    // to show how local variable is not accessible outside of
    // the scope of the function. Also, in 4), to show how variables
    // which are placed on stack memory are automatically
    // allocated and deallocated.
    // Tips: run it with the debugger line-by-line and follow-up
    // what happens with local variable a.
int nummerTo(int var) {
    int a{3};
    a += var;
    return a;
}

int main() {
    // 1) Pointers, "content of"(*) and "address of"(&) operators

    // Pointers are variables which contain address to another variable.
    // They give us possibility to access memory at a very low level.

	// An example of a pointer declaration:
	// pointer pHeltall is declared as a variable of the type
	// "pointer to int". Note '*' after the data type 'int'.
	// We can also write the same declaration as:
	// int *pHeltall{nullptr};
	// where '*' is written right in front of the pointer variable name.
	// These two ways of declaring a pointer variable are equivalent and
	// are matter of taste.
	//
	// As with all variables, it is a good programming practice to
	// initialise the pointer variable when we declare it. Pointer
	// variables are initialised by value 'nullptr' (null pointer).
	// If a pointer is equal to nullptr, it is considered to contain
	// an invalid address. For example, if the following if-statement:
	// if (pHeltall == nullptr)
	// returns true, the content of the pointer pHeltall is not a valid
	// address.
    int* pHeltall{nullptr};
	// An example of a declaration of a pointer to a double value.
    double* pReeltall{nullptr};
    int variabel1{0};
    int variabel2{5};
    double variabel3{0.0};

	// Operator '&' is called 'address of' operator. It returns the address
	// of a variable on which it is applied. In the example below, it returns
	// the address of a variable named 'variabel1' which is of the built-in
	// type int. Since pHeltall is a pointer to int value, it is possible to
	// assign the address of int variable to it. After the line of code below
	// is run, pHeltall will contain the address of variable variabel1.
    pHeltall = &variabel1;
	// When we write to the terminal the content of pointer pHeltall, as is
	// done by the line below, we see the address of variable variabel1.
	// Note that we use hexadecimal format to write the address. Addresses are
	// usually big numbers so by writing them in the hexadecimal format we achieve
	// clarity and easier overview of the memory locations and areas which are used.
    cout << "Peker inneholder addressen til variabel 1: " << hex << pHeltall << dec << endl; 

    // Pointers can be used to access values in the memory through the address
    // they contain. For that purpose they use operator '*' - a "content of" operator.

	// In the following example we can see that the value of variabel1
	// whe nwritten to the terminal is equal to the value of the content of the
	// memory location to which the pointer pHeltall contains an address (this is
	// because of the assignmet: pHeltall = &variabel1;
    cout << "Verdi av variabel1 er: " << variabel1 << endl;
	// The line below will write to the terminal the content of the memory
	// location to which pointer pHeltall points.
    cout << "Verdi av variabel1 gjennom peker er: " << *pHeltall << endl;

    // Pointers can also be used to change to value of the memory location of
    // which they contain the address.

	// In the following example the value of variable1 is changed via pointer
	// pHeltall which contains its address. You can read it as "the content of
	// memory location of which pHeltall contains the address is to be set
	// equal to 100".
    *pHeltall = 100;
    cout << "Verdi av variabel1 er: " << variabel1 << endl;
    cout << "Verdi av variabel1 gjennom peker er: " << *pHeltall << endl;

    // We can assign the address of another int variable to pointer pHeltall since
    // it is of type 'int*' (pointer to int):
    pHeltall = &variabel2;
    // ... but we cannot assign the address of a variable of another type than int
    // to a pointer declared as 'int*':
    // pHeltall = &variabel3; // this line will cause compile error

    // An example of assigning an address of a double variable to a pointer
	// to double value.
    pReeltall = &variabel3;
    *pReeltall = 3.14;
    cout << "Reeltall er: " << variabel3 << endl;

    // In the same way we define pointers to variables which are of user-defined types

	// An example of a declaration of a pointer to a user-defined type
	// class Qbit (see the top of this file for the class Qbit definition).
    Qbit* pQbit{nullptr};
    Qbit variabelQ{1};
	// An example of assigning an addres sof the object of type Qbit
	// to the pointer pQbit:
    pQbit = &variabelQ;
    	// An example of how we can get access to the public members of the class
	// Qbit to which pQbit points to:
	// a) directly from the pointer pQbit with operator '->' member of, where
	// '->' in this case refers to a member function getStat() of the object
	// of which pQbit contains the address
    cout << "Kaller medlemsfunksjon gjennom peker: " << pQbit->getStat() << endl;
    	// b) by dereferencing the pointer, i.e., applying the 'content of' / '*'
	// operator and then using the usual way of accessing the member of an object
	// by ".". This is possible since the dereferenced pointer / the content of the
	// address kept by the pointer / is the object of type class Qbit.
    cout << "Kaller medlemsfunksjon gjennom innholdet til peker: " << (*pQbit).getStat() << endl;

    // 2) Pointer arithmetic
    // C++ allows the content of the pointer variables to be changed by
    // applying certain operations as, for the example below, an increment
    // by 1. If the two values written to the terminal are compared, it can be seen that
    // the content of the pointer pHeltall is greater by 4 after it has been incremented
    // by 1 (by operation ++). The difference of 4 comes from the fact that an int variable
    // is stored in 4 bytes in the memory (for the OS and platforms we use in the course) and
    // since pHeltall is declared to be a pointer to int value, its increment by 1 will
    // increase the address by 4 (bytes).
    cout << "Peker inneholder addressen: " << hex << pHeltall << dec << endl;
    pHeltall++;
    cout << "Etter inkrement inneholder peker addressen: " << hex << pHeltall << dec << endl;

    // The same holds for the objects of the user-defined type. Note that in this
    // way we can see how much memory an object of the type of some user-defined
    // type (class, struct) takes up in the memory.
    cout << "Peker inneholder addressen: " << hex << pQbit << dec << endl;
    pQbit++;
    cout << "Etter inkrement inneholder peker addressen: " << hex << pQbit << dec << endl;

    // 3) Scope
    // Each name (of variable, function, user-defined data type etc) is valid within
    // a certain range or scope. While within its scope, the name gives a meaningful
    // access to a variable, function, user-deined data-type etc, but once out-of-scope
    // the name becomes invalid, it does not give access to any of the existing elements
    // of the program.
    // We have already seen how names can lose their meaning:
    // - think of local variables within a function! After the function returns, none of
    // the local variables can be accessed, any reference to their names outside
    // the scope of the function would produce a compiler error.
    // - think of class definitions! When we were defining a class, all members
    // within the block marked by braces after class <class_name> were within
    // the scope of that class, but for function definitions which were written
    // in a separate .cpp file we were using <class_name>:: qualifier to explicitly
    // say that the function belongs to the scope defined by that class (which is also
    // named - therefore, the namespace).
    // - think of enum class types! you have to used the qualifier with the enum class
    // name in front of each member if you want to use it!
    //
    // In general, the space between two matching braces is a so-called block scope.
    // That means that the block of statements after if-statement, for-, while-,
    // do-while loop also is a scope within which if new variables are declared,
    // they will also be valid only within that block of statements.
    // We can also make a block scope just by using two matching braces,
    // as in the example below:
    {
            // Definition of variable 'a' within the block scope:
        int a{7};
            // 'a' is valid within this block of statements - it is
            // an integer of value 7.
        cout << "Min a er: " << a << endl;
        for (int i = 0; i < a; i++) {
                // for-loop has introduced the scope within
                // which all the names valid in the higher level
                // scope are visible plus newly introduced variables
                // like counter variable 'i'.
            cout << i << "  ";
        }
        cout << endl;
            // The line below will cause a compiler error - 'i' is out of scope!
        //cout << i << endl;
    }
        // The line below will cause a compiler error - 'a' is out of scope!
    //cout << "a utenfor blokk-skop: " << a << endl;

        // An example of scope defined by a function. Run the line
        // below with a debugger and step-in (F11) to see what happens
        // with local variables on the function call stack.
    variabel2 = nummerTo(variabel1);
    cout << variabel2 << endl;

    // 4) Allocations and deallocations
    // - operators new and delete, new[] and delete[]

    // Beside stack memory, which we saw the use of for the example
    // of a function call, there is another type of memory - so called
    // heap memory (or free store in the pensum book PPP) which is
    // much larger than stack memory. Locations and ranges of locations
    // from this memory area can be obtained by a program in a runtime
    // by using certain C++ operators. The steps are as follows:
    // 1. Running program asks operating system (OS) for some amount of memory
    //    - use operators new or new[]
    // 2. OS checks if that amount is available and if it is, it marks
    //    the free memory as not-free (so other programs / applications
    //    cannot get valid access to it) and returns a pointer to the first
    //    byte of the allocated memory to the program.
    // 3. Program runs further, uses the allocated memory as long as it
    //    needs it
    // 4. When there is no further need for the allocated memory, the program
    //    notifies OS that it can free the memory
    //    - use operators delete or delete[]
    // 5. OS frees memory which it has kept for the program until this point
    //    and it can give it to use for other purposes (other programs)
    // Because this process happens while the program is running, dynamically
    // interacting with OS, this kind of memory allocation is called
    // dynamical allocation.

        // Declaration and initialisation of a pointer which we intend
        // to use to store the address of the first free byte of dynamically
        // allocated memory. Note how we initialise it to nullptr. In that
        // way we can test if the value - address returned by the OS is valid
        // if it becomes not equal nullptr.
    int* pDyn{nullptr};
        // Operator 'new' followed by the data type notifies OS
        // that we need a variable of that data type allocated on
        // the heap memory. In the line below, we ask OS for one integer
        // variable on the heap memory, the address of which is stored in
        // pointer pDyn.
    pDyn = new int;
        // To test if we got a valid pointer we can perform the check:
        // if (pDyn == nullptr) <handle_the_error_in_some_way>
        // (more about how to do this coming in lectures in few weeks,
        // for now we assume it is a valid address returned)

        // By dereferencing a pointer we can see which value is stored
        // in the newly allocated integer value on the heap:
    cout << "Allokert minnet inneholder ett heltall verdi: " << *pDyn << endl;
        // And we can use the pointer to that memory location to
        // access that value and do something with it - for example,
        // set the value to be equal 5:
    *pDyn = 5;
        // We can check if the integer on the heap has changed its value:
    cout << "Allokert minnet er tildelt heltall verdi: " << *pDyn << endl;
        // We don't want to use this integer value any more so we notify
        // OS that it can deallocate the heap memory it has kept for it
        // and give the memory to be used by others. For this purpose
        // we use operator 'delete' followed by the name of the pointer
        // which stored the address of the dynamically allocated memory.
    delete pDyn; // !!!!! MEMORY LEAK if delete not called!
                // Here is one of the dangers we can cause by using
                // pointers and operators for dynamical allocations
                // properly. If we don't call operator 'delete' on
                // a pointer which has previously been assigned an address
                // value by 'new' operator, we cause a so-called memory
                // leakage. This means that OS keeps holding this memory
                // on the heap reserved even when our pointer goes out
                // of scope and we no longer can use that memory.
                // The situation is as if we stop at point 3. as listed
                // above. Lesson to take:
                // every 'new' must have a matching 'delete'
    // !!!!! DANGLING POINTER
        // here is yet another danger when using pointers: although
        // the memory is freed by the OS after calling delete on
        // on the pointer pDyn, it is still accessible - it is still
        // within the scope - and can be used as if it contains some
        // address (pDyn is declared as pointer to an integer!). However,
        // that value is no longer valid, it can point to some memory
        // location which has in the meantime been allocated for another
        // program, for example.
        // You can, for example, set it to nullptr and, if used later
        // in the program, always test it for being equal to nullptr!
    cout << "Etter frigjøring av minnet!!!: " << *pDyn << endl;
    // !!!!! MULTIPLE DELETE CALLS
        // Uncomment the line below and run it. What happens?
        // At this point we have already called delete on the pointer
        // pDyn so it contains some invalid address. If we call delete
        // again, as in the line below, we tell to the OS that the
        // address in this pointer can be freed - but OS does not know of
        // that address any longer (it has 'recycled' any memory previously
        // associated with this pointer and we don't have control over
        // the value which is stored in this pointer, it can be anything!)
    //delete pDyn;
    //cout << "Etter dobbelt frigjøring av minne!!!: " << *pDyn << endl;

    // We can dynamically allocate a range of values of the same type.
    // For that purpose we use operator 'new[]'. In the end, heap memory
    // is mainly used when we need larger amounts of memory allocated
    // (stack is usually not that big!) or when we don't know at compile
    // time how much memory we will need - that is made known while program
    // is running (variables to be allocated on the stack must be known
    // at compile time!).

        // The line below shows the syntax for operator new[] where we
        // specify how many elements (5) of type int we ask OS to allocate
        // and where we initialise this memory to 5 values within braces:
    //int* pArr = new int[5]{0, 1, 2, 3, 4};
        // And here is the example where we find out the number of elements
        // to be allocated - i.e., the size of the array of elements -
        // from the user, while program is running:
    int n{0};
    cout << "Input number of elements:" << endl;
    cin >> n;
    int* pArr = new int[n];
    //for (int i = 0; i < 5; i++) {
    for (int i = 0; i < n; i++) {
        // The following lines show how we can use pointers and
        // pointer arithmetic to access the elements of a dynamically
        // allocated array:
        *(pArr+i) = i;
        cout << "Heap verdi: " << pArr[i] << endl;
        cout << "Heap verdi: " << *(pArr+i) << endl;
        // !!!!! BEWARE: no check on how far in the memory we can go -
        // access through pointer does not check the end of array of elements
        // (compare it to vector type from C++ standard library and the
        // difference between .at() method and use of index operator []
        // to access elements).
    }
        // After using operator new[] to ask for allocation of a range
        // of values of the same type, we use operator delete[] to let
        // the OS know we don't need that memory any longer and it can
        // free it. Lesson to take:
        // every 'new[]' must have a matching 'delete[]'
    delete[] pArr;
    //cout << pArr[3] << endl;
        // !!!!! double delete[] (the same applies as for 'delete')
    //delete[] pArr;

    // 5) Destructors
        // When objects are instantiated for the classes
        // which contain dynamically allocated members,
        // as for the example of an object of type class Cnot
        // (see the class definition at the top of this file!),
        // it is wise to define special functions called
        // destructors which will ensure that the allocated
        // resources from the heap are returned to the OS
        // when the object goes out of scope.
    Cnot cnObjekt;
        // 6) Copy-constructor
        // In such cases it is also wise to define a copy constructor.
        // Take a look at its definition for the example of class Cnot
        // at the top of this file.
        // The declaration below uses copy-constructor:
    Cnot cnObj2{cnObjekt};
        // Note that the line below is equivalent to the line above:
        // it is initialisation of of object, not a common assignment
        // and therefore it uses a copy constructor!
    //Cnot cnObj2 = cnObjekt;
    Cnot cnObj3;

    cnObjekt.writeQb(100);
    cout << cnObjekt.readQb() << endl;
        // 6b) overloading operator=() - copy assignment
        // See the definition of the class Cnot at the top of this file.
        // The assignment below uses overloaded operator=():
    cnObj3 = cnObjekt;

        // Since we have defined a destructor for class Cnot,
        // when the three objects of type Cnot go out of scope
        // at the end of main()-function / our program,
        // the destructors will be called and ask OS
        // to free the dynamically allocated resources by
        // the respective objects.
    return 0;
}
