#include <std_lib_facilities.h>
#include <AnimationWindow.h>
#include <widgets/Button.h>
#include <widgets/TextInput.h>
#include <widgets/DropdownList.h>

void modifyA(int& a) {

}

struct Printer {
    void say(std::string message) {
        std::cout << message << std::endl;
    }

    Printer() {
        std::cout << "Printer was created" << std::endl;
    }

    virtual ~Printer() {
        std::cout << "Printer was UTTERLY DESTROYED" << std::endl;
    }
};

// RAII: Resource Acquisition Is Initialisation

struct ChildPrinter : public Printer {
    std::unique_ptr<std::string> someMessage {new std::string{"Some message"}};

    ChildPrinter() {
        std::cout << "ChildPrinter was constructed" << std::endl;
    }

    virtual void doPrinting() {

    }

    virtual ~ChildPrinter() {
        std::cout << "ChildPrinter was destroyed" << std::endl;
    }
};

void usePrinter(std::unique_ptr<Printer> printerPointer) {

}

std::unique_ptr<Printer> createPrinter() {
    std::unique_ptr<Printer> pointer = make_unique<Printer>();
    return pointer;
}

TDT4102::TextInput input({300, 400}, 300, 30, "Change me");
void callbackFunction() {
    std::cout << "Text in box: " << input.getText() << std::endl;
}

int main() {
    TDT4102::AnimationWindow window;
    TDT4102::Button button({300, 300}, 100, 30, "Click me!");
    button.setCallback(&callbackFunction);
    window.add(button);

    
    input.setCallback(&callbackFunction);
    window.add(input);
    std::vector<std::string> vec{"Choice A", "Choice B", "Choice C"};
    TDT4102::DropdownList list({300, 500}, 100, 30, vec);
    window.add(list);


    window.wait_for_close();

    std::shared_ptr<Printer> sharedPrinter = make_shared<Printer>();
    std::cout << "Instance count: " << sharedPrinter.use_count() << std::endl;

    std::shared_ptr<Printer> anotherSharedPrinter = sharedPrinter;
    anotherSharedPrinter->say("Hello");
    std::cout << "Instance count: " << sharedPrinter.use_count() << std::endl;

    {
        std::shared_ptr<Printer> yetAnother = sharedPrinter;
        yetAnother->say("Hello");
        std::cout << "Instance count: " << sharedPrinter.use_count() << std::endl;

    }

    std::cout << "Instance count: " << sharedPrinter.use_count() << std::endl;


    std::unique_ptr<Printer> printer = createPrinter();

    // A constructor is called when the instance of the object is created
    // Its destructor is called at the end of the containing scope
    // (the } in the containing {}. Here it's the end of main())
    
    //Printer printer;
    
    
    std::unique_ptr<Printer> uniquePrinter;
    if(true) {
        // This is the manual way to allocate and deallocate memory
        // We want to avoid this as much as possible because it leads to memory leaks
        // Also, it's annoying having to remember to use delete
        // When we use new, the constructor of the object is called
        
        //Printer* anotherPrinter = new Printer();
        
        // When we use delete, the destructor is called

        //delete anotherPrinter;

        // When we allocate an array of objects, the constructors of each instance is
        // called when the array is created

        //Printer* manyPrinters = new Printer[10];
        //manyPrinters[4].say("Hello");

        // When we then use delete[] (which MUST be used for arrays),
        // the destructor is called for each instance

        //delete[] manyPrinters;

        // A unique_ptr is a structure which contains one pointer,
        // and guarantees there can only ever be one copy of that pointer
        // its destructor deletes the contained memory automatically
        //std::unique_ptr<Printer> print {new Printer()};

        //std::cout << print.get() << std::endl;
        // uniquePrinter.swap(print);
        // uniquePrinter->say("Hello there!");
        // usePrinter(std::move(uniquePrinter)); // Borrow

        // ChildPrinter print;
    }

    std::vector<std::unique_ptr<Printer>> vectorOfPrinters;
    vectorOfPrinters.emplace_back(new Printer());

    //std::unique_ptr<Printer> print {new Printer()};
    //std::unique_ptr<Printer> print = std::make_unique<Printer>();

    

    int a = 5;
    int& refToA = a;
    modifyA(a);

    int* pointerToA = &a;
    (*pointerToA)++;
    std::cout << *pointerToA << std::endl;


    for(int i = 0; i < 10; i++) {
        std::string* test = new std::string("Hello from the heap!");

        if(i == 3) {
            continue;
        }
        //delete test; // double free
        delete test;

        test->at(5); // dangling reference
    }
    

    return 0;
}