// Lecture 11, Destructors example, Dragana Laketic

#pragma once

#include "std_lib_facilities.h"

    // class Sattelite used as a helper to define a dynamically
    // alllocated data member of the class Houston.
class Sattelite {
    string name;
public:
    Sattelite() : name{"Apollo"} {} // Default constructor which sets
                                    // the data member name to "Apollo".
    Sattelite(string strName) : name{strName} {} // Overloaded constructor
                                    // which takes one parameter of type
                                    // string and assigns it to the data member name.
        // Simple member function which allows a user to read the value
        // of the data member name. Note that we can use const specifier
        // to underline that the data member will not be changed.
    string getName() const {
        return name;
    }
        // Simple member function which allows a user to set the value
        // of the data member name.
    void setName(string strName) {
        name = strName;
    }
};

class Houston {
        // class Houston has one data member and that is a pointer
        // to an object of type Sattelite.
    Sattelite *sattelite;
public:
        // Default constructor
    Houston() : sattelite{new Sattelite()} {} // Default constructor which
                                        // dynamically allocates an object
                                        // of type Sattelite and initialises
                                        // the pointer data member sattelite
                                        // with the address of the newly allocated
                                        // object.

        // Constructor which takes in one parameter of type string
    Houston(string strName) : sattelite{new Sattelite(strName)} {} // This constrctor
                                        // dynamically allocates an object of type
                                        // Sattelite and initialises it with the string
                                        // value which is passed as a parameter to this
                                        // constructor. The address of the newly allocated
                                        // and initialised object is assigned to the
                                        // class' data member sattelite.

        // Copy constructor
    //Houston(const Houston& initHouston) : sattelite{new Sattelite()} {} // creates new allocation but does not copy the value
    Houston(const Houston& initHouston) : sattelite{new Sattelite()} { // This copy constructor
                                        // takes in one parameter: a reference to the object
                                        // of the same type - class with which the object it
                                        // constructs will be initialised. Mark that we also
                                        // use const specifier to explicitly say that the object
                                        // used for initialisation will not be changed during
                                        // the construction of another object of this type.
                                        // In the initialisation list of this copy constructor
                                        // a new object of type Sattelite is dynamically allocated and
                                        // the data member sattelite of the object which is being
                                        // constructed is initialised with the address of this newly
                                        // allocated object.
        string initHoustonName = initHouston.readMySatteliteName(); // In the body of this constructor
                                        // we read the name of the object pointed to by data member
                                        // sattelite of the Houston object init Houston with which
                                        // we initialise the object constructed by this constructor.
        sattelite->setName(initHoustonName); // And then we assign this name to the newly allocated
                                        // object pointed to by data member sattelite.
                                        // This way of copying is called copy-by-value or DEEP COPY
                                        // as we not only copy the values (addresses) in pointer variable
                                        // but rather copy the value to newly allocated object. If we
                                        // just copied the values in sattelite data members, it would be
                                        // a SHALLOW COPY - the value of initHouston sattelite member would
                                        // be copied to sattelite data member of the new constructed object
                                        // so both objects would point to the same address in the memory.
    }
        // The copy constructor can be written as in the line below.
        // The definition above is more explanatory as of what happens during
        // construction by copying.
    // Houston(const Houston& initHouston) : sattelite{new Sattelite(initHouston.readMySatteliteName())} {}

        // Destructor
        // A special function called when the object goes out of scope and
        // and is being destroyed in memory. This is the place where we can
        // release resources allocated in the object's constructor (or during
        // object's lifetime): dynamically allocated memory, filestream objects and similar.
    ~Houston() {
        cout << "Destructor freeing memory!" << endl;
            // Here we let operating system to release the memory allocated
            // by new operator in the constructor to which the address has been
            // kept in sattelite data member.
        delete sattelite;
    }
        // We haven't tatlked about it in the class but just as a note and
        // recommendation: every time you write a copy constructor, write
        // an overloaded operator=(). This is useful in case you assign
        // an object which has some dynamically allocated resource(s) as
        // data member(s) to another object of the same type. For the same
        // reasons which hold for copy constructor, without overloaded
        // operator=() only shallow copy would be performed and two
        // objects would point to the same memory location.
    Houston& operator=(const Houston& assignHouston) {
        cout << "Assignment operator!" << endl;
        printMySatteliteName();
        sattelite->setName(assignHouston.readMySatteliteName());
        printMySatteliteName();

        return *this;
    }
    // Below are a few member function definitions which we used
    // to demonstrate properties of copy constructors and destructors.
    void printMySatteliteName() const {
        std::cout << "My sattelite's name: " << sattelite->getName() << endl;
    }
    void renameMySattelite(string strNewName) {
        sattelite->setName(strNewName);
    }
    string readMySatteliteName() const {
        return sattelite->getName();
    }
};
