// Lecture 11, Destructors example, Dragana Laketic

#include "std_lib_facilities.h"
#include "houston.h"

// Classes Sattelite and Houston which were used in the lecture
// are moved to a header file "houston.h" - renamed in this folder
// to "parallel2_destructors_houston.h".

int main() {
    // Pointers (dangers!) + Destructors & Copy Constructors
    Houston houston1; // Default constructor used to instantiate houston1.
    Houston houston2{"Sputnik"}; // Constructor which takes one string argument
                                // used to instantiate houston2.
    Houston houston5{houston1}; // copy-constructor used here
    Houston houstonX = houston1; // copy-constructor used here as well:
                                // note well that assignment here means initialisation!
    Houston* ptrHouston{nullptr}; // A pointer to an object of type class Houston.
                                // It is a good practice to initialise it to nullptr
                                // which means it contains an invalid address!

    houston1.printMySatteliteName(); // An object calls its method.
                                    // Name of the sattelite is determined in the
                                    // default constructor.
    ptrHouston = &houston2; // Pointer ptrHouston is assigned the address of houston2
                            // (operator & used on variable name which represents object
                            // of type class Houston.
    ptrHouston->printMySatteliteName(); // Example of accessing a member function
                                        // via pointer. Object of houston2 is instantiated
                                        // by calling a constructor with strin parameter
                                        // of value "Sputnik". It is this name that will be
                                        // written in the terminal after the method is called
                                        // via pointer.
    {
     // At this place we enter a block-scope denoted by braces.
     // All variables declared outside this block are valid within it as well.
     // However, variables declared / defined within the block will not be
     // valid outside the block - after we leave the closing brace "}".
        Houston houston3{"Rosetta"}; // here new Sattelite() is called - default constructor
        ptrHouston = &houston3; // Now pointer ptrHouston contains the address
                                // of houston3 object.
        ptrHouston->printMySatteliteName(); // Access of class method via pointer
    } // Here houston3 goes out of scope but delete sattelite was not being called!

    // We had an example of memory leakage - until we explicitly called delete
    // in the destructor ~Houston().and let operating system know that it can free
    // the memory.

    // Here ptrHouston is a dangling pointer!
    // It contains an address which is no longer a valid address -
    // object at that address is out of scope here.
    // But! Since it is declared as a pointer to an object of type Houston,
    // we can still call methods of that class:
    //ptrHouston->printMySatteliteName(); // Remember? This line caused a runtime error!

    houston1.printMySatteliteName();
    houston5.printMySatteliteName();
    cout << "Houston 5 changing name:" << endl;
    houston5.renameMySattelite("AlphaGO");
    // Before we wrote copy-constructor, the following two lines
    // were printing the same name, i.e., both houston1 and houston5
    // had their sattelite's names changed. This is because the member
    // sattelite is of type 'pointer to Sattelite' and it contains
    // the address of the object of type Sattelite. Without copy-constructor,
    // initialisation of object houston5 with houston2:
    // Houston houston5{houston1};
    // will be deduced so that the value of data member 'sattelite'
    // is copied and that value is an address to Sattelite object.
    // That is, both houston5 and houston1 will point to the same
    // Sattelite object in the memory.
    // After we wrote a copy constructor, where we defined the way how this
    // initialisation will be done, each object of type Houston keeps
    // a different pointer, different addresses to their respective Sattelite
    // (see how we defined the copy-constructor).
    houston1.printMySatteliteName();
    houston5.printMySatteliteName();

    return 0;
}
