// Lecture 11, Repetition example, Dragana Laketic

#pragma once

#include "std_lib_facilities.h"

// Abstract class:
// pure virtual function
// protected constructor

class Point {
protected: // What happens when you remove access protected
            // and make coord1 and coord2 private data members as by default?
    double coord1;
    double coord2;
public:
    virtual void printMyCoordinates() = 0; // An example of a pure virtual function.
                                        // If a class has at least one pure virtual
                                        // function, it is not possible to instantiate
                                        // an object of the type of that class. We say
                                        // that such class is abstract. We cannot have
                                        // objects - instances of an abstract class in
                                        // our program.
                                        // Another way to make a class abstract is to
                                        // make its constructor protected. In that way,
                                        // only its children - the classes which inherit it,
                                        // will be able to use its members.
                                        // Abstract classes are usually used for organising
                                        // hierachy of other classes, like in this simple
                                        // example of 2D points.
    virtual ~Point() {}; // virtualisation demands virtual destructor!
                        // Remember to add virtual destructor to the
                        // parent abstract class
};

class CartesianPoint : public Point {
public:
        // Default constructor:
        // Notice how CartesianPoint has no data members
        // of its own, but it inherits data members of its
        // parent class Point. Since they are declared
        // protected: in class Point, the classes which inherit
        // from Point can have direct access to them as in the
        // following examples of constructors:
    CartesianPoint() {
        coord1 = 0.0;
        coord2 = 0.0;
    }
    CartesianPoint(double x, double y) {
        coord1 = x;
        coord2 = y;
    }
    virtual void printMyCoordinates() {
        cout << "CartesianPoint coordinate: (x=" << coord1 << ", y= " << coord2 << ")" << endl;
    }
};

class PolarPoint : public Point {
public:
    PolarPoint() {
        coord1 = 0.0;
        coord2 = 0.0;
    }
    PolarPoint(double r, double phi) {
        coord1 = r;
        coord2 = phi;
    }
    virtual void printMyCoordinates() {
        cout << "PolarPoint coordinates: (radius=" << coord1 << ", angle=" << coord2 << ")" << endl;
    }
};
