#include <iostream>
#include <filesystem>
#include <fstream>
#include <vector>

struct Pizza {
    std::vector<std::string> toppings;
};

std::ifstream openFile(const std::filesystem::path& path) {
    std::ifstream fileStream {path};
    if(!fileStream) {
        std::cerr << "Could not open file: " + path.string() << std::endl;
    }
    return fileStream;
}

int readNumberOfPizzas(std::ifstream& fileStream) {
    int pizzaCount = 0;
    fileStream >> pizzaCount;
    return pizzaCount;
}

Pizza readNextPizza(std::ifstream& fileStream) {
    std::string line;
    std::getline(fileStream, line);

    Pizza pizza;
    std::stringstream lineStream {line};
    while(lineStream) {
        std::string topping;
        lineStream >> topping;
        pizza.toppings.push_back(topping);
    }
    return pizza;
}

std::vector<Pizza> parsePizzaFile(const std::filesystem::path& fileToRead) {
    std::ifstream pizzaFile = openFile(fileToRead);
    int numPizzas = readNumberOfPizzas(pizzaFile);

    std::vector<Pizza> pizzas;
    for(int pizzaIndex = 0; pizzaIndex < numPizzas; pizzaIndex++) {
        Pizza pizza = readNextPizza(pizzaFile);
        pizzas.push_back(pizza);
    }

    return pizzas;
}

void printPizzas(const std::vector<Pizza>& pizzas) {
    for(int pizzaIndex = 0; pizzaIndex < pizzas.size(); pizzaIndex++) {
        std::cout << "Pizza " << (pizzaIndex + 1) << ": ";
        const Pizza& pizza = pizzas.at(pizzaIndex);
        for(int toppingIndex = 0; toppingIndex < pizza.toppings.size(); toppingIndex++) {
            std::cout << pizza.toppings.at(toppingIndex) << ", ";
        }
        std::cout << std::endl;
    }
}

int main() {
    const std::filesystem::path fileToRead = "pizzas.txt";
    
    std::vector<Pizza> pizzas = parsePizzaFile(fileToRead);

    printPizzas(pizzas);
    return 0;
}