// Lecture 12, Example 1: GUI, Dragana Laketic

#include "std_lib_facilities.h"
#include "AnimationWindow.h"
#include "widgets/Button.h"
#include "widgets/TextInput.h"
#include "widgets/DropdownList.h"

// Examples of basic use of graphical user interface (GUI)
// implemented with graphicla elemets - widgets from the
// AnimationWindow library.

// NOTE: Example 1a) refers to the case when we open an instance
// of a default AnimationWindow.

/* 1a)
    // Example of a callback function. This function handles the event
    // "button-clicked", i.e., the code written within the function is
    // run when a button is clicked. Note that this function is assigned
    // to a particular variable of type TDT4102::Button within main().
void callbacButton() {
    static int gang{0}; // Here we can also see how static variables
                        // are used: when declared 'static', a variable
                        // does not lose its value when we go out of the
                        // scope of the function. It preserves its place
                        // in memory and the content of that memory location.
    gang++;
    cout << "Button clicked " << gang << " times" << endl;
}
*/

    // An example of a user-defined class of a window which
    // is derived from the basic classAnimationWindow. We usually
    // use windows derived from some basic window class in our GUI
    // programs since that gives us freedom to define how the window
    // will look and how each of its graphical elements will behave.
class MyWindow : public TDT4102::AnimationWindow {
        // We define widgets - graphical elements of the window as
        // data members of the window class. In this example we have
        // all three widgets: Button, TextInput and DropdownList as
        // elements of the window. Note how all three widgets follow
        // a similar pattern of parameters: first the coordinates of
        // the top left corner of the widget within the window, then
        // the width of the widget followed by its height. Following
        // parameters depend on the type of the widget: a button
        // will display the provided string as its name, a text box
        // will contain the text provided by the string, and the drop-down
        // meny will contain a list of items provided as a vector of strings
        // as its parameter.
        // Take a look at AnimationWindow documentation for a more
        // thorough description of these graphical elements.
    TDT4102::Button knapp{{100, 100}, 200, 100, "ClickMe!"};
    TDT4102::TextInput tekst{{400, 100}, 300, 100, "Write something here"};
    vector<string> frukt{"Eple", "Drue", "Plomme"};
    TDT4102::DropdownList lista{{100, 300}, 400, 100, frukt};
        // Callback functions (event handlers) which are assigned to
        // the graphical elements - data members of this class - are
        // provided as function members or methods of the class.
    void callbackKnapp() {
        cout << "Knappen clicked!" << endl;
    }
    void callbackTekst() {
        cout << "Teksten: " << tekst.getText() << endl;
    }
    void callbackLista() {
        cout << "Frukt: " << lista.getValue() << endl;
    }
public:
    MyWindow() {
            // We add widgets to the window within the constructor of our
            // user-defined window class. For that purpose add()-method
            // inherited from the base class AnimationWindow is used.
        add(knapp);
        add(tekst);
        add(lista);
            // We assign callback functions to particular widgets of
            // the window within window constructor as well. We use
            // widgets' setCallback() method. However, when this method
            // is called for the derived window class we cannot just
            // use the address of the function - we need to use the bind()
            // method which takes two parameters: the first one is the address
            // of the function we want to set as a callback (remember that in
            // this case we also have to provide the namespace where it is defined
            // and that is the name of our class!) and the second parameter
            // is always the pointer 'this', a pointer to the instance of the class at hand.
        knapp.setCallback(std::bind(&MyWindow::callbackKnapp, this));
        tekst.setCallback(std::bind(&MyWindow::callbackTekst, this));
        lista.setCallback(std::bind(&MyWindow::callbackLista, this));
    }
};

int main() {
        // An example of an event-driven programming:
        // a widow of the type we have defined is opened and
        // all the functionality in the program is implemented through
        // callback functions assigned to different graphical elements
        // of the window.
    MyWindow win;

    win.wait_for_close();

/*1a)
        // A simple example of how graphical user interface is implemented
        // with the possibilities provided by AnimationWindow library.
    TDT4102::AnimationWindow win; // Declaration of a default AnimationWindow.
    TDT4102::Button knapp{{100, 100}, 200, 100, "ClickMe!"}; // Definition of
                                // a graphical element in the window - a widget.
                                // AnimationWindow library provides three types of
                                // widgets: button (Button), text input field
                                // (TextInput) and drop-down meny (DropdownList).
                                // Pay attention to the header files which must be
                                // included when you want to use these in your
                                // program (see the top of this file, include section).
                                // Note how the definition of a widget is done: first
                                // we provide a pair of coordinates whihc correspond to
                                // the top left corner of the widget within the window,
                                // ({100, 100})then the width of the widget (200) and
                                // the widget's height (100). The last parameter, string
                                // "ClickMe!" refers to the text which will be shown on
                                // the button when it is created in the window.
                                // Check AnimationWindow documentation for wider description!

    win.add(knapp); // Calling method add() of the AnimationWindow class with the widget
                    // sent as its parameter results in the widget being added to the window
                    // and shown as defined.
    knapp.setCallback(&callbackButton); // The behaviour of the program is defined
                                // by the events produced when a user interacts with
                                // the graphical elements in the window. When an event
                                // happens with a certain graphical element in the window,
                                // the code defined within the so-called callback function
                                // is run. Callback functions are also called even-handlers
                                // for that reason.
                                // Widgets from the AnimationWindow library use their method
                                // setCallback() for that purpose. As a prameter, this method
                                // uses the address of the function which will be run when
                                // an event with the corresponding widget happens.

    win.wait_for_close(); // Usual way of keeping the window open until we explicitly close it.
*/
    return 0;
}
