// Lecture 13, Example 1: templates, Dragana Laketic

#pragma once
#include "std_lib_facilities.h"

// Templates enable the creation of classes and functions which have
// the same interface for different data types. The interface is
// written only once with the data type(s) passed in as parameters.
// Beside one or more data type parameters, there can also be values of
// some other type passed in to a template - most often it is integer
// values.
// When a template class or a template function is used in the program,
// we need to provide parameters - data type(s) and/or values - for the
// concrete class or function. When compiler comes across a templated
// class or function in the prgram file, it instantiates the corresponding
// template with exactly those parameters provided in the program. It is
// worth noting that this type-resolution and template instantiation
// happens at compile time.
// We keep template definitions - template class, template class member functions
// and template functions - within a header file. This is because it is only a template
// and the instantiated classes and functions, those which will be actually used
// in the program, are made by the compiler at compilation time.

    // 1) An example of a class template
    // With template we use a keyword 'template' followed by
    // keyword 'typename' and some name (as of our choice) between
    // less than and greater than signs. The name of a typename parameter
    // can be whatever you choose but often it is 'T' or some variation
    // of it for data types.
template<typename T>
class Point{
        // Where ever there is 'T' in the template class definition
        // there will be a concrete data type set by the compiler when
        // it comes across our invocation of the templated class with
        // that concrete parameter passed between '<' and '>'.
    T x;
    T y;
public:
    Point(T xx, T yy) : x{xx}, y{yy} {}
    T getX() const {return x;}
    T getY() const {return y;}
    void printMe() {
        cout << "(" << x << ", " << y << ")" << endl;
    }
};

    // 2) An example of a template class which takes more than one parameter
template<typename T1, typename T2>
class ChoppedPoint {
    T1 x;
    T2 y;
public:
    ChoppedPoint(T1 xx, T2 yy) : x{xx}, y{yy} {}
    void printMe() {
        cout << "(" << x << ", " << y << ")" << endl;
    }
};

    // 3) An example of a function template
template<typename TT> // note: here we use TT (our free choice)
                    //after 'typename'
void PrintValue(TT var) {
    cout << "Template function: " << endl;
    cout << var << endl;
}

    // 4) An example of integer value as a template parameter
    // (compare with std::array<> where we also pass in an integer value
    // as a template parameter)
template<typename T, int N>
class Collection {
    T* collection;
public:
    Collection() {
        cout << "Constructor of template<typename T, int N> class Collection" << endl;
        collection = new T[N];
    }
    ~Collection() {
        cout << "Destructor of template<typename T, int N> class Collection" << endl;
        delete[] collection;
    }
}
