#include <iostream>
#include <vector>
#include "Point2D.h"
#include "Point2DVirtual.h"

int main() {
    Point2D point{2, 3};
    CartesianPoint2D pointC{10, 20};
    PolarPoint2D pointP{1, 3};

    // Example of polymorphism
    // Derived classes call their version of the function printCoordinates()
    // - they override base class' version of the same function
    point.printCoordinates();
    pointC.printCoordinates();
    pointP.printCoordinates();

    // Example of runtime polymorphism - virtualisation
    // Base class Point2DVirtual has function printCoordinates()
    // defined as virtual.
    std::vector<Point2DVirtual*> points;

    points.reserve(12);

    for (int i = 0; i < 12; i++) {
        if (i % 4 == 0) {
            points.push_back(new Point2DVirtual);
        } else if (i % 3 == 0) {
            points.push_back(new CartesianPoint2DVirtual);
        } else {
            points.push_back(new PolarPoint2DVirtual);
        }
    }
    
    for (int i = 0; i < 12; i++) {
        // virtualisation method invoked when the functions are called
        // via pointer to the base class
        points.at(i)->printCoordinates(); // In this line points.at(i) is of type
                                    // "pointer to an instance (object) of class Point2dVirtual".
                                    // We use operator "->" to get the member function
                                    // printCoordinates() via pointer.
    }
    // and remember to deallocate elements of vector points:
    for (int i = 0; i < 12; i++) {
        delete points.at(i);
    }
    return 0;
}

//------------------------------------------------------------------------------
