#include "std_lib_facilities.h"

constexpr int NO_ERROR = 0; // we can have many error codes defined at some place
                            // in the program to be used as return values of functions
                            // to inform the caller about the state of the function
int incrementValues(int n, int* a, int* b);

int main() {
    // examples of pointer declarations
    int a{0}; // variable a of type 'int' initialised to value 0
    int* ptrInt; // variable ptrInt of type 'pointer to integer', not initialised

    cout << "Variable a initial value: " << a << endl;
    ptrInt = &a; // example of operator '&' - 'address of'
    // variable ptrInt now contains the address of integer variable a
    *ptrInt = 10; // example of use of operator '&' - 'content of' / pointer dereferencing
    // the value stored in variable a is now changed
    cout << "Variable a current value: " << a << endl;

    // example of using pointers for receiving more than one return value
    // from a function
    int firstNum{10};
    int secondNum{100};
    int returnStatus;
    cout << "Initial values, before calling incrementValues:" << endl
        << "firstNum: " << firstNum << "   secondNum: " << secondNum << endl;
    returnStatus = incrementValues(5, &firstNum, &secondNum);
    cout << "After calling incrementValues:" << endl
        << "firstNum: " << firstNum << "   secondNum: " << secondNum << endl; 
    if (returnStatus == NO_ERROR) { // example how we can use return value of the function
                                    // to pass info on error or exception (much C-like style)
        cout << "incrementValues run with no errors." << endl;
    }

    // example of dynamical allocation of matrix elements (Assignment 8)
    const int matrixSize{3};
    int** matrix;
    matrix = new int*[matrixSize]; // Variable matrix is a dynamically allocated array of
                                // matrixSize pointers to pointers to integer variables
    for (int i = 0; i < matrixSize; i++) {
        matrix[i] = new int[matrixSize]; // Each element of dynamically allocated matrix
                                        // is assigned a value of a pointer to a dynamically
                                        // allocated array of matrixSize integers
        // *(matrix + i) = new int[matrixSize]; // equivalent to: matrix[i] = new int[matrixSize];
    }
    for (int i = 0; i < matrixSize; i++) {
        for (int j = 0; j < matrixSize; j++) {
            matrix[i][j] = i+j;
        }
    }
    // write the values of the matrix and check if they are correct (row+column)
    for (int i = 0; i < matrixSize; i++) {
        for (int j = 0; j < matrixSize; j++) {
            cout << "m[" << i << "][" << j << "] = " << matrix[i][j] << "  ";
        }
        cout << endl;
    }

    // remember to deallocate memory with operator delete[]
    // for each allocation with new[], there has to be a corresponding delete[]
    for (int i = 0; i < matrixSize; i++) {
        delete[] matrix[i];
    }
    delete[] matrix;

    return 0;
}

int incrementValues(int n, int* a, int* b) {
    for (int i = 0; i < n; i++) {
        (*a)++;
        (*b)++;
    }

    return NO_ERROR;
}