#include <iostream>
#include <stack>
#include <queue>

template<typename T>
class Qubit {
    T x;
    T y;
    T z;
public:
    Qubit(T a, T b, T c) : x{a}, y{b}, z{c} {}
    void PrintQubit() {
        std::cout << "This Qubit:" << std::endl 
            << "X: " << x << std::endl
            << "Yi: " << y << std::endl
            << "Z: " << z << std::endl;
    } 
};

template<typename T>
void Print3Times(T& val) {
    std::cout << "Printing " << val << " " << "3 times:" << std::endl;
    for (int i = 0; i < 3; i++) {
        std::cout << val << " ";
    }
    std::cout << std::endl;    
}

template<typename T, int N>
void PrintNTimes(T& val) {
    std::cout << "Printing " << val << " " << N << " times:" << std::endl;
    for (int i = 0; i < N; i++) {
        std::cout << val << " ";
    }
    std::cout << std::endl;
}

class Point2D {
    int x;
    int y;
public:
    Point2D() : x{0}, y{0} {}
    Point2D(int a, int b) : x{a}, y{b} {}
    friend std::ostream& operator<<(std::ostream& outstream, const Point2D& pt);
};

std::ostream& operator<<(std::ostream& outstream, const Point2D& pt) {
    outstream << "(" << pt.x << ", " << pt.y << ")" << std::endl;
    return outstream;
}

int main() {
    Qubit<int> qubit1{0, 0, 1};
    Qubit<double> qubit2{0.7, 0.7, 0.7};

    qubit1.PrintQubit();
    qubit2.PrintQubit();

    int num1{10};
    int num2{5};
    Print3Times(num1); // example of template argument deduction
    PrintNTimes<int, 3>(num2);
    Point2D point{2, 3};
    PrintNTimes<Point2D, 2>(point); // we need to have operator<<() overloaded for Point2D!

    // STL data structures
    // The most used container / data structure from STL library is by far vector<>.
    // We have already seen many examples of its use so here comes an example with
    // stack and queue data structures. Stack is a LIFO (Last In First Out) and queue
    // a FIFO (First In First Out) data structure. This is reflected in how method pop()
    // removes elements from each of them as illustrated in the example below:
    // You don't have to know all of the data structures from STL! stack and queue are
    // given just as examples.
    std::stack<int> stack1;
    std::queue<int> queue1;

    for (int i = 0; i < 10; i++) {
        stack1.push(i);
        queue1.push(100*i);
    }
    while (!(stack1.empty() || queue1.empty())) {
        std::cout << stack1.top() << std::endl;
        std::cout << queue1.front() << std::endl;
        stack1.pop();
        queue1.pop(); 
    }
}