#include "Snake.h"

Snake::Snake(int gridWidth, int gridHeight, int initSize): snake(){
    /*
        Constructor of snake: initialize the snake vector at a random position within the grid
        Restricted to horisontal alignment only
    */
    // Random number machinery
    std::random_device rd;     // Only used once to initialise (seed) engine
    std::default_random_engine engine(rd());    // Random-number engine used

    std::uniform_int_distribution<int> uniW(initSize,gridWidth-initSize); // Head of snake  - x-pos 
    std::uniform_int_distribution<int> uniH(0,gridHeight); // Head of snake - y-pos 
    TDT4102::Point p = {uniW(engine), uniH(engine)};
    snake.push_back(p);

    // Add the rest of the body points, linearly behind the head
    for (int i = 0; i<initSize-1; ++i){
        snake.push_back(TDT4102::Point{snake.at(i).x-1, snake.at(i).y});
    }

    
}

void Snake::updateSnake(TDT4102::Point nextHead, bool didEat){
    /*
        Updated the length of the snake, if the food is eaten
        Translate the whole snake one step further 
    */
   
    if (didEat) {
        snake.push_back(TDT4102::Point{});      // This point is arbitrary as it will take value of the old "tail" point anyways
    }

    // Translate the snake one step, begin from the tail 
    // Assign each block the value of the block in front of it
    // Except the head, which must be assigned separately to "nexthead" (which is dependent on direction)
    for (int i = snake.size() - 1; i > 0; --i) {
        snake[i] = snake[i - 1];  // Shift body points (except head)
    }
    
    snake.at(0) = nextHead;    // Handling head separately
}

